%%%%% ps3 question 2: risk averse job search with saving

clearvars;
close all;
clc;

%%%%% economic parameters

alpha = 1;          %% CRRA (=1/IES)
beta  = 0.5;        %% time discount factor
delta = 0.05;       %% job destruction rate
b     = 0.40;       %% unemployment benefits
R     = (1/beta);   %% asset return

%%%%% wage distribution

mu    = -0.125;
sigma = 0.75;

%%%%% numerical parameters

max_iter = 1500;     %% maximum number of iterations
tol      = 1e-7;     %% treat numbers smaller than this as zero

%%%%% grid for wage distribution

nw       = 33;       %% number of nodes for quadrature (shocks)

% quadrature nodes and weights
[ew, ww] = qnwunif(nw,1e-9,1-1e-9);
ew       = mu+sigma*norminv(ew,0,1); 
wgrid    = exp(ew);
wmin     = min(wgrid);
wmax     = max(wgrid);

%%%%% grid for savings

na       = 33;        %% number of breakpoints for a grid
curv     = 0.99;          %% (curv = 0 log-spaced, curv = 1 linear)
amin     = 0.5*wmin/(1-beta) + eps;
amax     = 2*wmax/(1-beta);
agrid    = nodeunif(na, amin.^curv, amax.^curv).^(1/curv);

%%%%% put in a structure to pass to other functions

parameters.alpha = alpha;
parameters.beta  = beta;
parameters.delta = delta;
parameters.b     = b;
parameters.mu    = mu;
parameters.sigma = sigma;

parameters.wgrid = wgrid;
parameters.ww    = ww;

%%%%% setup state space using CompEcon tools

fspace0   = fundef({'spli',agrid}); % for unemployed, vbar(a) only

Phi0      = funbas(fspace0);        % matrix of collocation basis vectors 
                                    % Phi_{ij} = phi_j(k_i)
grid0     = funnode(fspace0);       % nodes where we solve the problem                                                                        
agrid0    = grid0;                  % extra 2 points for 3rd-order spline

fspace1    = fundef({'spli',agrid},...
                   {'spli', wgrid}); % for employed, v(a,w)

grid      = funnode(fspace1);       % nodes where we solve the problem
Phi1      = funbas(fspace1);        % matrix of collocation basis vectors 
                                   % Phi_{ij} = phi_j(k_i) 
agrid     = grid{1};               % extra 2 points for 3rd-order spline
wgrid     = grid{2};               % extra 2 points for 3rd-order spline

amin      = agrid(1);
amax      = agrid(end);

wmin      = wgrid(1); 
wmax      = wgrid(end);

%%%%% bounds on consumption

cmin     = tol;
cmax     = R*amax+wmax - amin;

%%%%% form collection of states

s        = gridmake(grid); % ns-by-2 matrix where ns=na*nw
ns       = size(s,1);        

a        = s(:,1);
w        = s(:,2);

%%%%%

if alpha==1
    
    u = @(c) log(c);
    
else 
    
    u = @(c) (c.^(1-alpha)-1)./(1-alpha);
    
end

%%%%% step #1: to get good starting point, solve as if always employed

c1 = (1-beta)*a;
v1 = u(c1)/(1-beta);
k1 = Phi1\v1;                      % implied collocation coefficients

k0 = ones(numel(agrid0));

for i=1:max_iter

c1 = solve_brent('rhs_bellman_always',s,parameters,k1,k0,fspace1,fspace0,cmin,cmax,tol);
v1 = rhs_bellman_always(c1,s,parameters,k1,k0,fspace1,fspace0);

k1new = Phi1\v1;

error = max(abs(k1new-k1));

fprintf('%4i %6.2e \n',[i, error]);

if error<tol, break, end

%%%%% if not converged, update and try again

k1 = 0.9*k1new+0.1*k1;

end

c0 = agrid0;
v0 = u(c0)/(1-beta);
k0 = Phi0\v0;

%%%%% step #2: solve allowing for unemployment

for i=1:max_iter

% if employed

c1    = solve_brent('rhs_bellman_employed',s,parameters,k1,k0,fspace1,fspace0,cmin,cmax,tol);
v1    = rhs_bellman_employed(c1,s,parameters,k1,k0,fspace1,fspace0);
k1new = Phi1\v1;

error1 = max(abs(k1new-k1));

% if unemployed
c0    = solve_brent('rhs_bellman_unemployed',agrid0,parameters,k1,k0,fspace1,fspace0,cmin,cmax,tol);
v0    = rhs_bellman_unemployed(c0,agrid0,parameters,k1,k0,fspace1,fspace0);
k0new = Phi0\v0;

error0 = max(abs(k0new-k0));

error  = max(error1,error0);

fprintf('%4i %6.2e \n',[i, error]);

if error<tol, break, end

%%%%% if not converged, update and try again

k1 = 0.5*(k1new+k1);
k0 = 0.5*(k0new+k0);

end

%savefile='ps3_question2.mat';      
%save(savefile);

%%%%% reshape solution

Na = numel(agrid);
Nw = numel(wgrid);

V1 = reshape(v1,Na,Nw); 
C1 = reshape(c1,Na,Nw); 

N0 = numel(agrid0);

V0 = reshape(v0,N0,1); 
C0 = reshape(c0,N0,1); 


selection = (1:1:Nw);

N         = numel(selection);

P         = brewermap(50,'*BuGn');

figure(1)
set(gcf,'DefaultLineLineWidth', 2);
axes('ColorOrder',P,'NextPlot','replacechildren')
plot(agrid,V1(:,selection),'LineWidth',2)
set(get(gcf,'CurrentAxes'),'FontSize',16,'LineWidth', 1.5)
ylabel('value function $v(a,w)$','Interpreter','latex')
xlabel('asset level, $a$','Interpreter','latex')
set(gca,'DefaultTextInterpreter', 'latex','TickLabelInterpreter','latex')
t = title('$v(a,w)$');
set(t,'Interpreter','latex');
%axis([0 120 -10 50])
box on


figure(2)
set(gcf,'DefaultLineLineWidth', 2);
axes('ColorOrder',P,'NextPlot','replacechildren')
plot(agrid,C1(:,selection),'LineWidth',2)
set(get(gcf,'CurrentAxes'),'FontSize',16,'LineWidth', 1.5)
ylabel('consumption function $c(a,w)$','Interpreter','latex')
xlabel('asset level, $a$','Interpreter','latex')
set(gca,'DefaultTextInterpreter', 'latex','TickLabelInterpreter','latex')
t = title('$c(a,w)$');
set(t,'Interpreter','latex');
%axis([0 120 0 12])
box on

figure(3)
set(gcf,'DefaultLineLineWidth', 2);
hold on
plot(wgrid,V0(4)*ones(numel(wgrid),1),'-','color',rgb('Red'),'LineWidth', 2.5)
plot(wgrid,V1(4,:),'-','color',rgb('Green'),'LineWidth', 2.5)
plot(wgrid,V0(12)*ones(numel(wgrid),1),'--','color',rgb('Red'),'LineWidth', 2.5)
plot(wgrid,V1(12,:),'--','color',rgb('Green'),'LineWidth', 2.5)
set(get(gcf,'CurrentAxes'),'FontSize',16,'LineWidth', 1.5)
ylabel('value function','Interpreter','latex')
xlabel('wage offer, $w$','Interpreter','latex')
set(gca,'DefaultTextInterpreter', 'latex','TickLabelInterpreter','latex')
t = title('$\bar{v}(a)$ against $v(a,w)$');
set(t,'Interpreter','latex');
l = legend('$\bar{v}(a)$','$v(a,w)$ for given $a$','Location','Best');
set(l,'Interpreter','latex');
box on

figure(4)
set(gcf,'DefaultLineLineWidth', 2);
axes('ColorOrder',P,'NextPlot','replacechildren')
hold on
plot(agrid0,V0,'rd-','LineWidth',2)
plot(agrid,V1(:,(1:1:Nw)),'LineWidth',1)
plot(agrid0,V0,'rd-','LineWidth',2)
hold off
set(get(gcf,'CurrentAxes'),'FontSize',16,'LineWidth', 1.5)
ylabel('value function','Interpreter','latex')
xlabel('asset level, $a$','Interpreter','latex')
set(gca,'DefaultTextInterpreter', 'latex','TickLabelInterpreter','latex')
t = title('$\bar{v}(a)$ against $v(a,w)$');
set(t,'Interpreter','latex');
l = legend('$\bar{v}(a)$','$v(a,w)$ for various $w$','Location','Best');
set(l,'Interpreter','latex');
%axis([0 120 -10 50])
box on




